/*  BREAKEMSoft GRAPH'EM Graphics Library v1.0 */

#define SET_SPRITE_SIZE(w,h) {sprite_width=w; sprite_height=h;}

#define retrace while(inp(0x3DA) & 0x08) {} while(!(inp(0x3DA) &  0x08)){}

typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned short int byte;

/* mode defines */
#define MODE13h 0x13
#define TEXT_MODE 0x03

/* palette defines */
#define PAL_MSK 0x3C6
#define PAL_GET 0x3C7
#define PAL_DAT 0x3C9
#define PAL_NUM 0x3C8

/* mouse defines */
#define H 1
#define V 2
#define POS 3
#define SHOW 4
#define HIDE 5

#define sprite_h         16        /* default h */
#define sprite_w         16        /* default w */

#define SPRITE_DEAD       0
#define SPRITE_ALIVE      1
#define SPRITE_DYING      2

typedef enum boolean {false,true} bool;

/*/////////////////////////////// pcxlib structures ///////////////////////*/
typedef struct RGB_palette_type
{
  unsigned char red;
  unsigned char green;
  unsigned char blue;
} RGB_pal, *RGB_pal_ptr;

typedef struct pcx_type
{
  RGB_pal pal[256];
  uchar __far *buffer;
} pcx, *pcx_ptr;

/*///////////////////////////////// sprite and image structures ///////////*/
typedef struct sprite_type
{
  int x,y;
  int x_old, y_old;
  int x_vel, y_vel;
  int width,height;
  int anim_clock;
  int anim_speed;
  int motion_speed;
  int motion_clock;
  uchar __far *frames[200];
  int cur_frame;
  int num_frames;
  int state;
  uchar __far *background;
  void __far *extra_data;
} sprite, *sprite_ptr;

typedef struct image_typ
{
  int w,h;
  uchar __far *buffer;
} image, *image_ptr;

/*//////////////////////// functions ///////////////////////*/

#ifdef __cplusplus
 extern "C" {
#endif

/* draw functions */
void draw_box(int x1,int y1,int w,int h,byte color);
void setpal(int num,int r,int g,int b);
void getpal(int num,int *r,int *g,int *b);
void blit_string(char *string,int x,int y,int color);
void blit_char (char ch, int xc,int yc,int color);
void grad_string(char *string,int x,int y,int color, int inc);
void grad_char (char ch, int xc,int yc,int color, int inc);
void plotpixel (int x, int y, int color);
int  getpixel (int x, int y);
void fillscr (int color);
void set_mode (int mode);
void vline (int y1,int y2, int x, int color);
void hline (int x1,int x2, int y, int color);
void fillbox (int x1,int y1,int x2,int y2, int color);
void mouseset (int type,int x,int y);
void returnmouse(int *x,int*y,int*b);
void mouseinit(void);
void draw_line(int xo, int yo, int x1,int y1, unsigned char color);

/* pcxlib functions */
int  PCX_init (pcx_ptr picture);
void PCX_load (char *filename,pcx_ptr picture, int load_pal);
void PCX_put (pcx_ptr picture);
void PCX_del (pcx_ptr picture);
void PCX_save (char *filename);
void PCX_pixel (pcx_ptr picture);

/* sprite functions */
int  sprite_init(sprite_ptr sprite,int frames);
void sprite_delete(sprite_ptr sprite);
void draw_sprite (sprite_ptr sprite);
void sprite_PCX_get (sprite_ptr sprite1, pcx_ptr pcx1,int frame,int grab_x,int grab_y);
void behind_sprite (sprite_ptr sprite);
void erase_sprite (sprite_ptr sprite);
int  sprite_collide(sprite_ptr sprite_1, sprite_ptr sprite_2,int smaller);
void pixel_sprite (sprite_ptr sprite, int inout);
void create_mouse (sprite_ptr sprite);
int  image_init(image_ptr image,int wid,int hieght);
void image_put (image_ptr image,int px,int py);
void image_get (image_ptr image,int gx,int gy);
void image_del (image_ptr image);
void image_PCX_get(image_ptr image,pcx_ptr pcx,int grab_x, int grab_y);

/*////////////////// double_buffer /////////////////////////*/

#define copy_buffer memcpy((uchar __far *)&video[0],(uchar __far *)&double_buffer[0],64000L);

int  create_buffer(void);
void delete_buffer(void);
void plotpixel_DB(int x,int y,int color);
int  getpixel_DB(int x,int y);
void hline_DB (int x1,int x2, int y, int color);
void vline_DB (int y1,int y2, int x, int color);
void behind_sprite_DB(sprite_ptr sprite);
void erase_sprite_DB(sprite_ptr sprite);
void draw_sprite_DB(sprite_ptr sprite);
void draw_cell_DB(sprite_ptr sprite);
void fade_in (uchar __far *pal);
void fade_out(void);
void blit_char_DB(char c,int xc,int yc,int color);
void blit_string_DB(char *string,int x,int y,int color);
void create_mouse_DB (sprite_ptr sprite);
void PCX_put_DB (pcx_ptr pcx);
void grad_string_DB(char *string,int x,int y,int color, int inc);
void grad_char_DB(char c,int xc,int yc,int color, int inc);
void fillbox_DB (int x1,int y1,int x2,int y2, int color);
void scale_sprite(sprite_ptr sprite,float scale);
void draw_box_DB(int x1,int y1,int w,int h,byte color);
void image_put_DB (image_ptr image,int px,int py);
void draw_line_DB(int xo, int yo, int x1,int y1, unsigned char color);

#ifdef __cplusplus
 }
#endif

/*///////////////// externals //////////////////////////////*/

extern unsigned int sprite_height;
extern unsigned int sprite_width;

extern unsigned char __far *video;
extern unsigned char __far *char_set;

extern unsigned char __far *double_buffer;


